/*
 *   This is part of the Reker - Library.
 *   Copyright (C) 1999  Frank Reker <frank@reker.net>
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "reker.h"
#include "buffer_types.h"


int bufconvert (BUFFER buf, R_BUFFER_TYPE dtid) {
	_r__dummy_buffer_type * p;
	_r__mem_buffer_type * pms, * pmd;
	_r__file_buffer_type * pfs, * pfd;
	_r__cont_buffer_type * pcs, * pcd;
	_r__buffer_type * q;
	FILE * f;
	char * s;
	int c, retval = 1;
	if (!_r__buffer) return 0;
	if (!buf) return 0;
	p = (_r__dummy_buffer_type *) buf;
	switch (dtid) {
	case R_NULL_BUFFER:
	    switch (p->id) {
	    case R_NULL_BUFFER: return 1;
	    case R_MEM_BUFFER:
		retval *= bufclean (buf);
		p->id = R_NULL_BUFFER;
		p->oid = R_NULL_BUFFER;
		break;
	    case R_FILE_BUFFER:
		pfs = (_r__file_buffer_type *) buf;
		fclose (pfs->f);
		unlink (pfs->filename);
		free ((void *) pfs->filename);
		pfs->filename = NULL;
		pfs->len = 0;
		pfs->overflow = 0;
		p->id = R_NULL_BUFFER;
		p->oid = R_NULL_BUFFER;
		break;
	    case R_CONT_BUFFER:
		pcs = (_r__cont_buffer_type *) buf;
		free ((void *) pcs->buffer);
		pcs->buffer = NULL;
		pcs->bufferlen = 0;
		pcs->len = 0;
		pcs->overflow = 0;
		p->id = R_NULL_BUFFER;
		p->oid = R_NULL_BUFFER;
		break;
	    default: return 0; }
	    break;
	case R_MEM_BUFFER:
	    switch (p->id) {
	    case R_NULL_BUFFER:
		p->id = R_MEM_BUFFER;
		p->oid = R_MEM_BUFFER;
		break;
	    case R_MEM_BUFFER:
		return 1;
	    case R_FILE_BUFFER:
		pmd = (_r__mem_buffer_type *) buf;
		pfs = (_r__file_buffer_type *) buf;
		if ((p->flags & R_BUFFLAG_AUTOCONVERT) &&
			(p->len > p->max_mem_size)) {
			p->oid = R_MEM_BUFFER;
			break; }
		q = (_r__buffer_type *) malloc (sizeof (_r__buffer_type));
		q->buf = bufgetstr (buf);
		q->next = NULL;
		fclose (pfs->f);
		unlink (pfs->filename);
		free ((void *) pfs->filename);
		pmd->buffer = q;
		pmd->bufferend = q;
		p->id = R_MEM_BUFFER;
		p->oid = R_MEM_BUFFER;
		break;
	    case R_CONT_BUFFER:
		pcs = (_r__cont_buffer_type *) buf;
		pmd = (_r__mem_buffer_type *) buf;
		q = (_r__buffer_type *) malloc (sizeof (_r__buffer_type));
		q->buf = realloc (pcs->buffer, (size_t) strlen (pcs->buffer));
		q->next = NULL;
		pmd->buffer = q;
		pmd->bufferend = q;
		p->id = R_MEM_BUFFER;
		p->oid = R_MEM_BUFFER;
		break;
	    default: return 0; }
	    break;
	case R_FILE_BUFFER:
	    switch (p->id) {
	    case R_NULL_BUFFER:
		pfd = (_r__file_buffer_type *) buf;
		s = strdup (R_TMP_FILE);
		mktemp (s);
		f = fopen (s, "w+");
		pfd->f = f;
		pfd->filename = s;
		p->id = R_FILE_BUFFER;
		p->oid = R_FILE_BUFFER;
		break;
	    case R_MEM_BUFFER:
		pms = (_r__mem_buffer_type *) buf;
		pfd = (_r__file_buffer_type *) buf;
		s = strdup (R_TMP_FILE);
		mktemp (s);
		f = fopen (s, "w+");
		for (q=pms->buffer; q; q=q->next)
			fprintf (f, "%s", q->buf);
		while (pms->buffer) {
			q=pms->buffer->next;
			free ((void *) pms->buffer->buf);
			free ((void *) pms->buffer);
			pms->buffer = q; }
		pms->bufferend = NULL;
		pfd->f = f;
		pfd->filename = s;
		s = NULL;
		p->id = R_FILE_BUFFER;
		p->oid = R_FILE_BUFFER;
		break;
	    case R_FILE_BUFFER:
		return 1;
	    case R_CONT_BUFFER:
		pcs = (_r__cont_buffer_type *) buf;
		pfd = (_r__file_buffer_type *) buf;
		s = strdup (R_TMP_FILE);
		mktemp (s);
		f = fopen (s, "w+");
		fprintf (f, "%s", pcs->buffer);
		free (pcs->buffer);
		pfd->f = f;
		pfd->filename = s;
		s = NULL;
		p->id = R_FILE_BUFFER;
		p->oid = R_FILE_BUFFER;
		break;
	    default: return 0; }
	    break;
	case R_CONT_BUFFER:
	    switch (p->id) {
	    case R_NULL_BUFFER:
		p->id = R_CONT_BUFFER;
		p->oid = R_CONT_BUFFER;
		break;
	    case R_FILE_BUFFER:
		if ((p->flags & R_BUFFLAG_AUTOCONVERT) &&
			(p->len > p->max_mem_size)) {
			p->oid = R_CONT_BUFFER;
			break; }
		pcd = (_r__cont_buffer_type *) buf;
		pfs = (_r__file_buffer_type *) buf;
		if (p->flags & R_BUFFLAG_GROW_FAST)
			for (c = R_BUFFER_PAGE; c <= p->len; c *= 2);
		else	c = (p->len / R_BUFFER_PAGE) * R_BUFFER_PAGE +
			(p->len % R_BUFFER_PAGE) ? 1 : 0;
		s = (char *) malloc ((size_t) c);
		bufprints (buf, s);
		fclose (pfs->f);
		unlink (pfs->filename);
		free ((void *) pfs->filename);
		pcd->buffer = s;
		pcd->bufferlen = c;
		s = NULL;
		p->id = R_CONT_BUFFER;
		p->oid = R_CONT_BUFFER;
		break;
	    case R_MEM_BUFFER:
		pcd = (_r__cont_buffer_type *) buf;
		pms = (_r__mem_buffer_type *) buf;
		if (p->flags & R_BUFFLAG_GROW_FAST)
			for (c = R_BUFFER_PAGE; c <= p->len; c *= 2);
		else	c = (p->len / R_BUFFER_PAGE) * R_BUFFER_PAGE +
			(p->len % R_BUFFER_PAGE) ? 1 : 0;
		s = (char *) malloc ((size_t) c);
		bufprints (buf, s);
		while (pms->buffer) {
			q=pms->buffer->next;
			free ((void *) pms->buffer->buf);
			free ((void *) pms->buffer);
			pms->buffer = q; }
		pms->bufferend = NULL;
		pcd->buffer = s;
		pcd->bufferlen = c;
		s = NULL;
		p->id = R_CONT_BUFFER;
		p->oid = R_CONT_BUFFER;
		break;
	    case R_CONT_BUFFER:
		return 1;
	    default: return 0; }
	    break;
	default: return 0; }
	return retval;
}
