/*
 *   This is part of the Reker - Library.
 *   Copyright (C) 1999  Frank Reker <frank@reker.net>
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "reker.h"
#include "buffer_types.h"


int bufcp (BUFFER dest, BUFFER src) {
	BUFFER help;
	_r__dummy_buffer_type * pd, * ps;
	_r__mem_buffer_type * pmd, * pms;
	_r__file_buffer_type * pfd, * pfs;
	_r__cont_buffer_type * pcd, * pcs;
	_r__buffer_type * qs;
	char * s;
	int c, retval = 1;
	if (!_r__buffer) return 0;
	if (!dest || !src) return 0;
	pd = (_r__dummy_buffer_type *) dest;
	ps = (_r__dummy_buffer_type *) src;
	if (dest == src) {
		help = bufopen (ps->id, NULL); /* NULL is ignored, if not file
							buffer */
		retval *= bufcp (help, src);
		retval *= bufmv (dest, help);
		retval *= bufclose (help); 
		return retval; }
	switch (pd->id) {
	case R_NULL_BUFFER:
	    switch (ps->id) {
	    case R_NULL_BUFFER: 
	    case R_MEM_BUFFER:
	    case R_FILE_BUFFER:
	    case R_CONT_BUFFER: return 1;
	    default: return 0; }
	    break;
	case R_MEM_BUFFER:
	    pmd = (_r__mem_buffer_type *) dest;
	    switch (ps->id) {
	    case R_NULL_BUFFER: return 1;
	    case R_MEM_BUFFER:
		pms = (_r__mem_buffer_type *) src;
		if (!pms->buffer) return 1;
		for (qs=pms->buffer; qs; qs=qs->next) 
			retval *= bufprint (dest, qs->buf);
		break;
	    case R_FILE_BUFFER:
		pfs = (_r__file_buffer_type *) src;
		if (!pfs->len) return 1;
		rewind (pfs->f);
		while ((c=fgetc (pfs->f)) != EOF) {
			s = (char *) malloc ((size_t) R_BUFFER_PAGE + 1);
			ungetc (c, pfs->f);
			if (!fgets (s, R_BUFFER_PAGE+1, pfs->f)) break; 
			s = realloc (s, (size_t) strlen (s) + 1);
			retval *= bufputstr (dest, s); }
		fseek (pfs->f, 0, SEEK_END);
		s = NULL;
		break;
	    case R_CONT_BUFFER:
		pcs = (_r__cont_buffer_type *) src;
		if (!pcs->len) return 1;
		retval *= bufprint (dest, pcs->buffer);
		break;
	    default: return 0; }
	    break;
	case R_FILE_BUFFER:
	    pfd = (_r__file_buffer_type *) dest;
	    switch (ps->id) {
	    case R_NULL_BUFFER: return 1;
	    case R_MEM_BUFFER:
		pms = (_r__mem_buffer_type *) src;
		if (!pms->len) return 1;
		for (qs=pms->buffer; qs; qs=qs->next)
			retval *= bufprint (dest, qs->buf);
		break;
	    case R_FILE_BUFFER:
		pfs = (_r__file_buffer_type *) src;
		if (!pfs->len) return 1;
		rewind (pfs->f);
		while ((c=fgetc(pfs->f))!=EOF) {
			if ((pfd->flags & R_BUFFLAG_MAXSIZE) &&
				(pfd->len == pfd->max_size)) {
				pfd->overflow++;
				break; }
			fprintf (pfd->f, "%c", (unsigned char) c);
			pfd->len++; }
		fseek (pfs->f, 0, SEEK_END);
		break;
	    case R_CONT_BUFFER:
		pcs = (_r__cont_buffer_type *) src;
		retval *= bufprint (dest, pcs->buffer);
		break;
	    default: return 0; }
	    break;
	case R_CONT_BUFFER:
	    pcd = (_r__cont_buffer_type *) dest;
	    switch (ps->id) {
	    case R_NULL_BUFFER: return 1;
	    case R_MEM_BUFFER:
		pms = (_r__mem_buffer_type *) src;
		if (!pms->len) return 1;
		for (qs=pms->buffer; qs; qs=qs->next)
			retval *= bufprint (dest, qs->buf);
		break;
	    case R_FILE_BUFFER:
		pfs = (_r__file_buffer_type *) src;
		if (!pfs->len) return 1;
		rewind (pfs->f);
		s = (char *) malloc ((size_t) R_BUFFER_PAGE + 1);
		while ((c=fgetc (pfs->f)) != EOF) {
			ungetc (c, pfs->f);
			if (!fgets (s, R_BUFFER_PAGE+1, pfs->f)) break; 
			retval *= bufprint (dest, s); }
		fseek (pfs->f, 0, SEEK_END);
		free ((void *) s);
		s = NULL;
		break;
	    case R_CONT_BUFFER:
		pcs = (_r__cont_buffer_type *) src;
		retval *= bufprint (dest, pcs->buffer);
		break;
	    default: return 0; }
	    break;
	default: return 0; }
	return retval;
}
