/*
 *   This is part of the Reker - Library.
 *   Copyright (C) 1999  Frank Reker <frank@reker.net>
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



/************************************************************************
 * Begin buffer definitions						*
 ************************************************************************/

#include <stdarg.h>
#include <stdio.h>

typedef void * BUFFER;


typedef enum {
	R_NO_BUFFER = 0,	/* not a buffer, only for error reports 
					and detection */
	R_NULL_BUFFER = 1,	/* a dummy buffer, that doesn't buffer 
					anything */
	R_MEM_BUFFER = 2,	/* a buffer, that uses a linked list of 
					strings as buffer */
	R_FILE_BUFFER = 3,	/* a buffer, that uses a temp file as buffer */
	R_CONT_BUFFER = 4	/* a buffer, that uses a large string, that 
					is enlarged if necessary */
	} R_BUFFER_TYPE;

typedef enum {
	R_BUFFLAG_NONE = 0,
	R_BUFFLAG_MAXSIZE = 1,		/* don't let the buffer become larger 
					than max_size */
	R_BUFFLAG_AUTOCONVERT = 2,	/* convert buffer to file buffer, when
					buffer becomes larger than 
					max_mem_size */
	R_BUFFLAG_NO_RECONVERT = 4,	/* normally the buffer is reconverted 
					to old type, if buffer becomes smaller
					then max_mem_size; this flag prevents
					reconversion */
	R_BUFFLAG_GROW_FAST = 8,	/* only interesting for continues 
					buffer: with this flag, the buffer is
					doubled if enlargend; normally it is
					extended by R_BUFFER_PAGE = 1024 */
	R_BUFFLAG_ALL = 0xffffffff
	} R_BUFFER_FLAGS;



extern BUFFER bufopen (R_BUFFER_TYPE tid, ...);


extern int bufsetflags (BUFFER buf, unsigned int _flags);


extern int bufunsetflags (BUFFER buf, unsigned int _flags);


extern int bufsetmaxsize (BUFFER buf, unsigned int _max_size);

	
extern int bufsetmaxmemsize (BUFFER buf, unsigned int _max_mem_size);


extern BUFFER bufinit (R_BUFFER_TYPE tid, ...);
	

extern int buftrunc (BUFFER buf, unsigned int length);
	

extern int bufoverflow (BUFFER buf);


extern int bufresetoverflow (BUFFER buf);


extern int bufprint (BUFFER buf, const char * str);


extern int bufprintf (BUFFER buf, const char * fmt, ...);


extern int bufvprintf (BUFFER buf, const char * fmt, va_list ap);


extern int bufnprintf (BUFFER buf, unsigned int num, const char * fmt, ...);


extern int bufvnprintf (BUFFER buf, unsigned int num, const char * fmt, va_list ap);


extern int buflprintf (BUFFER buf, const char * fmt, ...);


extern int bufvlprintf (BUFFER buf, const char * fmt, va_list ap);


extern int bufclean (BUFFER buf);


extern int bufclose (BUFFER buf);


extern int bufprintout (BUFFER buf, FILE * f);


extern unsigned int bufcount (BUFFER buf);


extern unsigned int buflen (BUFFER buf);


extern unsigned int bufgetflags (BUFFER buf);


extern unsigned int bufgetmaxsize (BUFFER buf);


extern unsigned int bufgetmaxmemsize (BUFFER buf);


extern R_BUFFER_TYPE bufkind (BUFFER buf);


extern R_BUFFER_TYPE bufoldkind (BUFFER buf);


extern char * bufgetstr (BUFFER buf);


extern char * bufprints (BUFFER buf, char * dest);


extern unsigned int bufevallen (BUFFER buf);
		

extern void * bufgetref (BUFFER buf);


extern int bufputstr (BUFFER buf, char * s);


extern int bufputc (BUFFER buf, const char c);


extern int bufmv (BUFFER dest, BUFFER src);


extern int bufcp (BUFFER dest, BUFFER src);
	
	
extern int bufcloseall ();


extern int bufconvert (BUFFER buf, R_BUFFER_TYPE dtid);





