/*
 *   This is part of the Reker - Library.
 *   Copyright (C) 1999  Frank Reker <frank@reker.net>
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "reker.h"
#include "buffer_types.h"


int buftrunc (BUFFER buf, unsigned int length) {
	_r__dummy_buffer_type	* p = NULL;
	_r__mem_buffer_type	* pm = NULL;
	_r__file_buffer_type	* pf = NULL;
	_r__cont_buffer_type	* pc = NULL;
	_r__buffer_type		* q, *q2;
	unsigned int i = 0, j;
	if (!_r__buffer) return 0;
	if (!buf) return 0;
	p = (_r__dummy_buffer_type *) buf;
	if (p->len <= length) return 1;
	switch (p->id) {
	case R_NULL_BUFFER: break;
	case R_MEM_BUFFER: 
		pm = (_r__mem_buffer_type *) buf;
		q = pm->buffer;
		while (1) {
			if (!q) return 0;
			j = strlen(q->buf);
			if ((i+j) > length) break;
			i += j;
			q = q->next; }
		realloc (q->buf, (size_t) length - i + 1);
		q->buf [length - i] = 0;
		while (q->next) {
			q2 = q->next->next;
			free ((void *) q->next->buf);
			free ((void *) q->next);
			q->next = q2; }
		pm->len = length;
		pm->bufferend = q;
		break;
	case R_FILE_BUFFER: 
		pf = (_r__file_buffer_type *) buf;
		rewind (pf->f);
		ftruncate (fileno(pf->f), (size_t) length);
		fseek (pf->f, 0, SEEK_END);
		pf->len = length;
		if (!(pf->flags & R_BUFFLAG_NO_RECONVERT) &&
			(pf->oid != R_FILE_BUFFER) &&
			(pf->max_mem_size >= pf->len)) 
			bufconvert (buf, pf->oid);
		break;
	case R_CONT_BUFFER: 
		pc = (_r__cont_buffer_type *) buf;
		if ((length == 0) && (pc->bufferlen != R_BUFFER_PAGE))
			pc->bufferlen = R_BUFFER_PAGE;
		else if (pc->flags & R_BUFFLAG_GROW_FAST) {
			while ((pc->bufferlen / 2) > (length +1)) 
				pc->bufferlen /= 2; 
			if (pc->bufferlen < R_BUFFER_PAGE) 
				pc->bufferlen = R_BUFFER_PAGE; }
		else 	pc->bufferlen = ((length + 1) / R_BUFFER_PAGE) * 
				R_BUFFER_PAGE + R_BUFFER_PAGE;
		pc->buffer = realloc (pc->buffer, (size_t) pc->bufferlen);
		pc->buffer [length] = 0;
		pc->len = length;
		break;
	default: return 0; }
	return 1;
}
